      program transq
**********************************************************************
*                                                                    *
*    PROGRAMME  TRANSQ              VERSION 1   October 1993         *
*    Simulation of traverse topographs                               *
*                                                                    *
*    General purpose program with strain function previously         *
*    computed and stored in files to be recalled                     *
*                                                                    *
*    See DEFW1      Version June 1993                                *
*        TRANSV     Version May 1993                                 *
*                                REFERENCE DEFV1 24/10/1985          *
*    REFERENCES :                                                    *
*                Y.EPELBOIN MAT.SCIENCE AND ENGINEERING              *
*                           73,1-43 (1985)                           *
*                C. CARVALHO & Y. EPELBOIN                           *
*                Acta Cryst.A49, 460-467  (1993)                     *
*                Acta Cryst.A49, 467-473  (1993)                     *
*                                                                    *
*    Y.EPELBOIN                                                      *
*                LABORATOIRE DE MINERALOGIE CRISTALLOGRAPHIE,        *
*                ASSOCIE AU CNRS,UNIVERSITE P.M.CURIE, Case 115,     *
*                75252 PARIS CEDEX05,FRANCE (33) 1+4427 5211         *
*                                                                    *
*                                                                    *
*      MAIN for program transq                                       *
*                                                                    *
*      This MAIN drives all the computation                          *
*                 - opens the input file and computes all the        *
*                   data common for all the integrations of          *
*                   Takagi equations   (function debut)              *
*                 - opens the input files for the strain and loads   *
*                   them in memory.  (function strain)               *
*                 - computes Takagi equations for each incidence     *
*                   plane.                                           *
*                   Many functions may run in parallel               *
*                 - write the data in the image file (function image)*
*                                                                    *
**********************************************************************
*
**********************************************************************
*    LAST UPDATE      First version                    October 1993  *
*                                                                    *
*                     End debugging                    10/12/1994    *
*                     Dummy arrays in include files    13/03/1994    *
*                     transposition array strain_val                 *
*                     optimization for cache in takagi 15/09/1995    *
*                     Wrong check array size in strain 26/09/1995    *
*                                                                    *
*                                                                    *
**********************************************************************
*
      include 'version.h'
*
*     parametres de debut
*
      include 'reseau.h'
*
*     parametres de strain
*
      include 'strain.h'
*
*     parametres de takagi
*
      include 'takagi.h'
      common /impc/titre
      character * 80 titre  
      common /impd/filestr
      character *40 filestr
      integer debut, takagi, strain, strain_load, image, store
      integer i, ir
      real total
      integer processor
*
*
*     variables de debut
*
      include 'debut_array.h'
*              real ttrv(ndim)
*              real q3(ndim),q4(ndim)
*              real s0x(0:ndim),s0z(0:ndim),shx(0:ndim)
*              real shz(0:ndim)
      include 'reseau_array.h'
*              real p(ndim),q(ndim),q1(ndim),q2(ndim)
*              real transv(ndim)
      real epais,ep1,ep2
      real bords0,constante
      integer nm 
*     variables de strain
*
      include 'strain_array.h'
*              integer  dimx(proc_max), offset(proc_max)
*              integer line_length(proc_max)
      integer  dimz, x_debut
      integer line_min,line_max,image_size,mul
      real stepx,stepz
      logical exists
*
*     Variables de strain_load
*
      include 'strain_load_array.h'
*              real strain_val(2*z_max,x_max)
*
*     Variables image
*
      include 'image_array.h'
*              real out(proc_max,int_max)
*
      if (int_max.gt.x_max) then
          write(*,*) 'inconsistent values for parameters '
          write(*,*) 'int_max = ',int_max,' x_max = ',x_max
          write(*,*) 'modify headers strain.h and takagi.h',
     &    ' and recompile'
          stop 'Error main transq'
      endif
*
*     Initialization of common data for integration
*
      ir = debut(epais,ep1,ep2,transv,
     &ttrv,nm,p,q,q1,q2,q3,q4,s0x,s0z,shx,shz,
     &bords0,constante)
      if (ir.ne.0) then
          write(15,*) 'Error diffraction initialization'
          close(unit=15)
          stop 'Error diffraction initialization'
      endif
*
*     Initialization of strain data for each incidence plane
*
      ir =  strain(dimx,dimz,offset,stepx,stepz,x_debut,
     &line_min,line_max,image_size,line_length,mul,exists,ep1,ep2,
     &epais)
      if (ir.eq.-1) then
          write(15,*) 'Error strain initialization'
          close(unit=15)
          stop 'Error strain initialization'
      endif
*
      write(*,1000)ver_1,ver_2,ver_3,ver_4,ver_5,ver_6,ver_7,
     &             ndim,npas,proc_max,x_max,
     &             z_max,int_max,image_max
      write(15,1000)ver_1,ver_2,ver_3,ver_4,ver_5,ver_6,ver_7,
     &              ndim,npas,proc_max,x_max,
     &              z_max,int_max,image_max
1000  format(//'Information about the program'/
     &         '-----------------------------'/
     &      'Version ref.'/7('       ',A/)/
     &      'Network: Max number of nodes :',I4,' (ndim)'/ 
     &      '         Max number of class :',I4,' (npas)'/
     &      '         see header reseau.h'/
     &      'Strain:  Max processors:      ',I4,' (proc_max)'/
     &      '         Max array x:         ',I4,' (x_max)'/
     &      '         Max array z:         ',I4,' (z_max)'/
     &      '         see header strain.h'/
     &      'Image:   Max intensity buffer ',I4,' (int_max)'/
     &      '         Max image buffer:    ',I4,' (image_max)'/
     &      '         see header takagi.h')
            total = (16*ndim + npas +4*proc_max + 2*z_max*x_max
     &              + int_max*ndim*15
     &              + 4*proc_max*int_max)*1.e-06
      write(*,1001) total
      write(15,1001) total
1001  format(/ 'Total space for arrays (Mwords) ',F10.3)
      write(*,1002)
      write(15,1002)
1002  format(/'Time coefficient (Mnodes)      processor #')
      do processor=1,line_max-line_min + 1
         write(*,1003)float(nm*nm)*0.5E-6*line_length(processor),
     &                  processor
         write(15,1003)float(nm*nm)*0.5E-6*line_length(processor),
     &                   processor
1003     format('    ',F10.1,'                 ',I3)
      enddo
      write(*,1004)
      write(15,1004)
1004  format(/)
      if (titre(1:4).eq.'test'.or.titre(1:4).eq.'TEST') then
            write(*,*) ' No calculation performed'
            write(15,*) ' No calculation performed'
            close(unit=15)
            stop 'Exit for data test'
      endif
*
*
*     The following loop must be modified to work
*     with a multiprocessor machine
*
CMIC$ do all private(strain_val,ir)
CMIC$1       shared(dimz,dimx,line_min,x_debut,mul,stepx,stepz,
CMIC$2              nm,p,q,q1,q2,q3,q4,s0x,s0z,shx,shz,constante,
CMIC$3              bords0,ttrv,transv,line_length,offset,
CMIC$4              image_size,out)
      do processor = 1, line_max-line_min + 1
*
*     Computation of Takagi equations
*
*   - 1 - Read strain data from file and load into memory of processor
*
          ir = strain_load(processor,strain_val,dimx,dimz,line_min)
          if (ir.ne.0) then
              write(*,*) 'Processor ',processor
              write(15,*) 'Processor ',processor
              write(15,*) 'Error loading strain'
              close(unit=15)
              stop 'Error loading strain'
          endif
*
*   - 2 - Compute partial derivative equation for one line of the image
*         using the strain data loaded by strain_load (1)
*         
          ir = takagi(processor,strain_val,dimz,
     &         x_debut,mul,stepx,stepz,nm,p,q,
     &         q1,q2,q3,q4,s0x,s0z,shx,shz,constante,bords0,
     &         ttrv,transv,line_length(processor),offset(processor),
     &         image_size,out)
          if (ir.ne.0) then
              write(*,*) 'Processor ',processor
              write(15,*) 'Processor ',processor
              write(15,*) 'Error Takagi equations'
              close(unit=15)
              stop 'Error Takagi equations'
          endif
      enddo
*
*    - 3 - Write image on disk
*
      ir = store(line_min,line_max,out,image_size,exists)
      if (ir.ne.0) then
           write(*,*) 'Error writing image'
           write(15,*) 'Error writing image'
           stop 'Error writing image' 
      endif
      write(15,1010)
      write(*,1010)
1010  format(///'End of calculation'/
     &          '------------------')
      write(*,1011)
1011  format(///)
      close(unit=15)
      stop 'TRANSQ Normal end'
      end
      
