      integer function strain(dimx,dimz,offset,stepx,stepz,x_debut,
     &line_min,line_max,image_size,line_length,mul,exists,
     &ep1,ep2,epais)
      character *40 strain_file
      common /impd/filestr
      character *40 filestr
      common /impe/image_file
      character *40 image_file
*
*     read strain data stored as one file for one incidence plane
*
*     Description of strain files as
*     - number of incidence planes
*     - generic name for the strain files
*     - step of sampling
*     For each file:
*     - dimensions of the array
*     - offset of data in image
*
*     Data
*     dimx(proc_max) x dimension of strain files Max. # proc_max
*     dimz           z dimension of strain files
*     offset         offset of strain data in the image Max. # proc_max
*     stepx, stepz   resolution steps in strain files
*     x_debut        Position of first calculated point in strain_file
*     line_min 
*     line_max       first and last lines in the image
*     image_size     Max # of pixels in a line
*     line_length    # of pixels in a line of the image
*     mul            resolution in the image is mul*stepx      
*     exists         logical for existence of an image file
*     ep1,ep2        input only, related to the thickness of the 
*                    crystal (see debut)
*     epais          input only, thickness of crystal
*      
      include 'strain.h'
      include 'strain_array.h'
*                 integer  dimx(proc_max), offset(proc_max)
*                 integer line_length(proc_max)
      integer dimz, x_debut
      integer line_min,line_max,image_size,mul
      real stepx,stepz,ep1,ep2,epais
      logical exists
*
      character * 45 file_1
      integer i,nxrep
      integer iin, ir, j, errcod, ix, iz, last_line
      integer proc_number,x_fin
      include 'takagi.h'
      include 'store.h'
*
      parameter (iin = 10)
*
*     data
*          proc_number: number of incidence planes (or processors)
*          proc_number = line_max -line_min +1
*          dimz, dimx : size of strain files: 2*dimz,dimx
*     the names of the files are filestr//1 -> line_number
*     Per file, one line with offset in the image 
*     in the mesh
*     Then each file is read as described in the program
*
      write(*,*) 'File for strain description? '
      read (*,'(A)') strain_file
      open (unit = iin, file = strain_file, status = 'old',
     &      iostat = errcod)
      if (errcod .ne. 0) then
          write (*,*) 'Error OPEN strain_data file ', strain_file 
          strain = errcod
          return
      endif
      read (unit=iin,fmt='(A)') filestr
      read(unit=iin,fmt=*) line_min, line_max,image_size
      proc_number = line_max - line_min +1
      if(proc_number.gt.proc_max) then
          write(*,*) 'too much incidence planes'
          write(*,*) 'proc_max =  ',proc_max
          write(*,*) 'modify header strain.h and recompile'
          write(*,*) 'WARNING: maximum value computer dependant'
          strain = -1
          return
      endif
      read(unit=iin,fmt=*) dimz,stepx,stepz,mul
      if(dimz.gt.z_max) then
          write (*,*) 'Max. size for strain files:'
          write(*,*) 'x = ',x_max,' z = ',z_max
          write(*,*) 'modify header strain.h and recompile'
          strain = -1
          return
      endif
      write(unit=15,fmt=2000)filestr,proc_number,stepx,stepz,dimz
      write(unit=*,fmt=2000)filestr,proc_number,stepx,stepz,dimz
2000  format(//'Strain characteristics'/
     &         '----------------------'//
     &         'generic name for data files:  ',A/
     &         'number of parallel processes:  ',I4/
     &         'step size along x (microns):   ',F7.2/
     &         'step size along z (microns):   ',F7.2/
     &         'dimension z for strain files:  ',I4)
      if (float(dimz)*stepz.lt.epais) then
          write(*,*) 'size z of strain inconsistent with thickness'
          write(*,*) 'check data and try again'
          strain = -2
          return
      endif
      write(unit=15,fmt=2003)
      write(unit=*,fmt=2003)
2003  format(
     &/'File #    Offset    dimension x   First    Last  # pixels')
*
      read(unit=iin,fmt='(A)') image_file
      x_debut=max(nint(ep1/stepx),0)
      do i = 1, proc_number
          read(unit=iin,fmt=*,end=1000) offset(i),dimx(i)
          if(dimx(i).gt.x_max) then
              write(*,*) 'Wrong x dimension for strain_val'
              write(*,*) 'File # ',i+line_min-1, 'x = ',dimx(i)
              write(*,*) 'Max. value = ', x_max
              write(*,*) 'Check header strain.h'
              strain=-1
              return
          endif
          x_fin=dimx(i) - nint(abs(ep2)/stepx+0.5)
*
          line_length(i) = (x_fin -x_debut+1)/mul
          offset(i)=nint(float(offset(i))/float(mul)) + 1
          if (line_length(i)+offset(i).gt.int_max) then
              write(*,*) 'line length exceeded = ',line_length(i)+
     &                    offset(i)
              write(*,*) 'maximum allowed int_max = ',int_max
              write(*,*) 'check header takagi.h and recompile'
              write(*,*) 'processor = ',i
              strain = -1
              return
          endif
          write(unit=15,fmt=2001)i+line_min-1,offset(i), dimx(i),
     &         x_debut,x_fin,line_length(i)
          write(unit=*,fmt=2001)i+line_min-1,offset(i), dimx(i),
     &         x_debut,x_fin,line_length(i)
2001  format(  '  ',I4,'   ',I4,'         ',I4,'        ',I4,
     &         '     ',I4,'     ',I4)
          if(dimx(i).lt.nint(abs(ep1-ep2)/stepx+0.5)) then
               write(*,*) 'dimension x of file # ',i+line_min
     &         -1,' too small'
               strain = -1
               return
          endif
          image_size = max(image_size,line_length(i)+offset(i))
*
*         limits of the image in each strain file      
*
      enddo
      close (unit = iin)
*
*     check consistency for image file
*
      inquire(file=image_file,exist=exists)
      if (exists.eqv..true.) then
*
*    File opened with wrong blocksize, just to read nxrep
*
           open (unit=20,file=image_file,status='old',
     &iostat=ir,access='direct',recl=80,form='unformatted')
           if(ir.ne.0) then
              write(*,*) 'unable to open ',image_file
              write(*,*) 'for sequential read'
              strain = -1
              return
           endif
           read(unit=20,rec=1) nxrep
           close(unit=20)
           if (nxrep.lt.image_size) then
              write(*,*) 'image size inconsistent with',
     &        ' previous calculation:'
              write(*,*) image_size,' instead of ',nxrep
              strain = -1
              return
           else
              image_size = nxrep
              write(*,2002) 
              write(15,2002) 
2002          format("Warning, image size changed to value",
     &               " in previous computation")
           endif
           open(unit=20,file=image_file,form='unformatted',
     &     status='old',access='direct',recl=mot*image_size,
     &     iostat=ir)
           if(ir.ne.0) then
              write(*,*) 'unable to open ',image_file,
     &        ' for direct access read'
              strain = -1
              return
           endif
           read(unit=20,rec=3)nxrep,last_line
           close(unit=20)
           if(last_line.lt.line_min -1) then
              write(*,*) 'first line in computation',
     &        ' inconsistent with previous calculation'
              write(*,*) 'last computed line ',last_line
              strain = -1
              return
           endif
      endif
*
      write(unit=*,fmt=2004)image_file,line_min,line_max,
     &image_size,float(mul)*stepx
      write(unit=15,fmt=2004)image_file,line_min,line_max,
     &image_size,float(mul)*stepx
2004  format(//'Image characteristics'/
     &         '---------------------'//
     &         'file name:            ',A/
     &         'first and last line   ',I3,' ',I3/
     &         'line length           ',I4,/
     &         'resolution (microns): ',F7.2)
      strain = 0
      return
*
*     Error offset
*
1000  continue
      write(*,*) 'EOF Offset description incomplete'
      write(15,*) 'EOF Offset description incomplete'
      strain = -1
      return
      end
