      integer function strain_load(processor,strain_val,
     &dimx,dimz,line_min)
      implicit none
*
*     load strain data into the memory of the 
*     processors
*
*     Modified for compatibility with parallel version for shared
*     memory processors
*
*     Data
*
*     processor        # of the running processor
*     strain_val       array of strain functions 
*                      du/dx  odd elements
*                      du/dz  even elements
*     dimz, dimx       dimension of arrays strain_val
*                      dimx: array size proc_max
*     line_min         first line in image
*
      include 'strain.h'
      common /impd/filestr
      character *40 filestr
      character *45 file_1
      integer i, j, line_min, ix, iz, ir
      include 'strain_load_array.h'
*            real strain_val(x_max,2*z_max)
      include 'strain_array.h'
*            integer  dimx(proc_max), offset(proc_max)
*            integer line_length(proc_max)
      integer processor, dimz
      integer iin,jjn
      parameter (iin=30)
*
*     processor: processor number: first = 1
*     line_min:  first line in image
*     strain_val:array of strain data
*
*     building files names: generic_name//line_number
*
      do j = 40,1,-1
          if(filestr(j:j).ne.' ') goto 10
      enddo
10    continue
      write(unit=file_1,fmt=2002)filestr(1:j),processor
     &      + line_min -1
2002  format(A,I3.3)

*      
*     building unit numbers to read the data
*     from 30 to 45
*
      jjn=iin+mod(processor,16)
      open(unit=jjn,form='unformatted', status='old',
     &      file=file_1, iostat=ir)
      if(ir.ne.0) then
          write(*,*) 'Cannot access strain value file ',file_1
          strain_load = -1
          return
      endif
      write(*,*) 'File ',file_1,' opened for unit ',jjn
      
*
*   strain data are stored as follow:
*   - perpendicular to the surface (first index varies first)
*   - x and z components following for each node
*     In this version symetric case only means that du1/dx du1/dz
*     only are known. For the general case du3/dx and du3/dz must
*     be known, i.e. 4 data per node
*
      do iz =1,2*dimz
         read(jjn,end=1100,iostat=ir) (strain_val(ix,iz),
     &        ix =1,dimx(processor))     
         if(ir.ne.0) then
             write(*,2004) processor,ir
2004         format('Error reading strain file # ',I4/'Code = ',I4)
             strain_load = -1
             return
         endif
      enddo
          close(unit=jjn)
*
*     here any data transformation such as derivatives 
*     from strain values, multiplication by constants ...
*     as long as they will be given in the arguments list
*
      strain_load = 0
      return
*     
*     size of strain files too small
*
1100  continue
      write(*,*) 'wrong size for strain file ',file_1
      write(*,*) 'read z = ',iz,' x = ',ix,' points'
      write(*,*) 'processor = ',processor
      strain_load = -1
      return
      end
