      integer function takagi(processor,strain_val,dimz,
     &        x_debut,mul,stepx,stepz,
     &        nm,p,q,q1,q2,q3,q4,s0x,s0z,shx,shz,constante,
     &        bords0,ttrv,transv,line_length,offset,
     &        image_size,out)
*
*     Compute the Takagi equations for one line in the image     
*     (One incidence plane)
*     To be launched in parallel on multiprocessors machines
*
******************************************************************
*                                                                *
*  Parallelisation of the computation of an image line           *
*                                                                *
******************************************************************
*
*     Data from function debut()  for the integration
*          from function strain for the deformation
*      
*     Data
*       
*     processor        # of running processor
*     strain_val       array of strain data
*     dimz             dimensions of array
*     stor             Array of computed intensity
*     x_debut          Coordinates of first and last points
*     mul              resolution factor
*     nm               number of nodes along s0 or sh
*     p,q              steps along s0 and sh
*     q1,q2            functions of q for computation of vectors R
*     q3,q4            Aditionnal functions for asymetric case (not in use)
*     bords0           intensity along s0
*     s0x,s0z,shx,shz  Components of position along s0 and sh
*     constante        constant for vector T
*     ttrv             absorbtion value per node
*     transv           step along the exit surface
*     line_length      Number of pixels to be computed
*     offset           offset of information in out
*     image_size       size of image (number of pixels per line
*
      include 'reseau.h'
      include 'strain.h'
      include 'takagi.h'
      integer processor,dimz,mul,x_debut,nm,offset
      include 'strain_load_array.h'
*                 real strain_val(x_max,2*z_max)
      include 'image_array.h'
*                 real out(proc_max,image_max)
      include 'reseau_array.h'
*                 real p(ndim),q(ndim),q1(ndim),q2(ndim)
*                 real transv(ndim)
      include 'debut_array.h'
*                 real ttrv(ndim)
*                 real q3(ndim),q4(ndim)
*                 real s0x(0:ndim),s0z(0:ndim),shx(0:ndim)
*                 real shz(0:ndim)
      real bords0,constante,stepx,stepz
*
      real intens(int_max,0:ndim),du1dx(int_max),du1dz(int_max)
      real x,z,delta_x,delta_z
      real delta_xz,delta_x1z,delta_xz1,delta_x1z1
      real stor(int_max)
      integer i_x,i_z,i_z2
      integer line_length,iplan,n,alpha,k,image_size
      real dstepz,dstepx,x_0,z_0
      complex Vn_2(int_max,0:ndim),Vn_1(int_max,0:ndim)
      complex Vn(int_max,0:ndim)
      complex S(int_max,ndim),R(int_max,ndim)
      complex T(ndim),R1(int_max,ndim)
*
      complex UN
      parameter (UN=(1.,0.))
*
*           z_0, x_0 coordinates of summit of Bormann fan
*
      x_0 = x_debut*stepx
      z_0 = (dimz-1)*stepz
      dstepx = 1./stepx
      dstepz = 1./stepz
*
*     Computation of intensity along a line in the image 
*     corresponding to one incidence plane
*     Initialization of intensity buffer for each calculation (intens)
*     and image line (stor)
*
      do iplan =1,line_length
          stor(iplan)=0.
          intens(iplan,0)=bords0
      enddo
*
*      initialisation V vectors
*      For the meaning of V, R, T, S ... vectors, see notebook IV p. 72
*      and following
*
      do iplan = 1,line_length
          Vn_2(iplan,0)= UN          
          Vn_1(iplan,0)= UN          
          Vn_1(iplan,1)= UN          
          Vn(iplan,0)= UN          
          S(iplan,1)= UN          
       enddo
*
*     Initialization of R1 vector for each point
*
       do alpha=1,nm
*
*         interpollating points 0,im
*
              x=x_0 - (shx(alpha)+shx(alpha-1))*0.5
              z=z_0-(shz(alpha)+shz(alpha-1))*0.5
*
*             definition of the cell where to interpollate
*
              i_x=int(x*dstepx)+1
              i_z=int(z*dstepz)+1
              delta_x=x*dstepx - float(i_x)-1.
              delta_z=z*dstepz - float(i_z)-1.
              delta_xz = delta_x * delta_z
              delta_x1z = (1.-delta_x) * delta_z
              delta_xz1 = delta_x * (1.-delta_z)
              delta_x1z1 = (1.-delta_x) * (1.-delta_z)
              i_z2 = i_z*2
*
*             compute du1/dx du1/dz  Warning change signs because 
*             axes changed from strain to diffraction
*             du/dx odd values, du/dz even values
*             du/dx = strain_val(i_x,2*i_z+1)
*             du/dz = strain_val(i_x,2*i_z)
*        
          do k=1,line_length
          du1dx(k) =-strain_val(i_x+(k-1)*mul,i_z2-1)*delta_xz
     &       - strain_val(i_x+(k-1)*mul+1,i_z2-1)*delta_x1z
     &       - strain_val(i_x+(k-1)*mul,i_z2+1)*delta_xz1
     &       - strain_val(i_x+(k-1)*mul+1,i_z2+1)*delta_x1z1

          du1dz(k) = - strain_val(i_x+(k-1)*mul,i_z2)*delta_xz
     &        - strain_val(i_x+(k-1)*mul+1,i_z2)*delta_x1z
     &        - strain_val(i_x+(k-1)*mul,i_z2+2)*delta_xz1
     &        - strain_val(i_x+(k-1)*mul+1,i_z2+2)*delta_x1z1
          enddo
*
*    For asymetric case du3/dx and du3/dz must be known
*    Add q3(alpha)*du3dx +q4(alpha)*du3dz in R1 expression
*    q3 and q4 are computed in this program to facilitate this upgrade
*
          do iplan =1,line_length
              R1(iplan,alpha)=cmplx(0.,q1(alpha)*du1dx(iplan)
     &                        +q2(alpha)*du1dz(iplan))
          enddo
       enddo      
*
*     End of initialization
*
*     Start main loop n through crystal
*     Computation of a triangle along a line "//" to the surface
*     n is the line number
*
          do n=2,nm
*
*         Computation of vector R
*
             do alpha=1,n
*
*            interpollating points 0,im
*
                 x=x_0 - s0x(n-alpha) - (shx(alpha)+shx(alpha-1))*0.5
                 z=z_0 - s0z(n-alpha) - (shz(alpha)+shz(alpha-1))*0.5
*
*                definition of the cell where to interpollate
*
                 i_x=int(x*dstepx)+1
                 i_z=int(z*dstepz)+1
                 delta_x=x*dstepx - float(i_x)-1.
                 delta_z=z*dstepz - float(i_z)-1.
                 delta_xz = delta_x * delta_z
                 delta_x1z = (1.-delta_x) * delta_z
                 delta_xz1 = delta_x * (1.-delta_z)
                 delta_x1z1 = (1.-delta_x) * (1.-delta_z)
                 i_z2 = i_z*2
*
*                compute du/dx du/dz  Warning change signs because 
*                axes changed from strain to diffraction (see above R1)
*                and change again sign, thus do not make any change
*                since complex part of R multiplied by -q1 and -q2
*                See notebook IV 77
*        
                 do k=1,line_length
                 du1dx(k) =
     &       strain_val(i_x+(k-1)*mul,i_z2-1)*delta_xz +
     &       strain_val(i_x+(k-1)*mul+1,i_z2-1)*delta_x1z +
     &       strain_val(i_x+(k-1)*mul,i_z2+1)*delta_xz1 +
     &       strain_val(i_x+(k-1)*mul+1,i_z2+1)*delta_x1z1
*
                 du1dz(k) =
     &       strain_val(i_x+(k-1)*mul,i_z2)*delta_xz +
     &       strain_val(i_x+(k-1)*mul+1,i_z2)*delta_x1z +
     &       strain_val(i_x+(k-1)*mul,i_z2+2)*delta_xz1 +
     &       strain_val(i_x+(k-1)*mul+1,i_z2+2)*delta_x1z1
                 enddo
*
*    For asymetric case du3/dx and du3/dz must be known
*    Add q3(alpha)*du3dx +q4(alpha)*du3dz in R expression
*
                 do iplan=1,line_length
                   R(iplan,alpha) = cmplx(1.,q1(alpha)*du1dx(iplan)
     &                   +q2(alpha)*du1dz(iplan))+ R1(iplan,alpha)
                 enddo
             enddo
*
*        Computation of vector T
*
             do alpha=1,n-1
                  T(alpha) = cmplx(p(n-alpha)*q(alpha)*constante,0.)
*
*     Computation of amplitude  (see notebook IV 73) for one point alpha
*     in line n for all image points iplan
*
                  do iplan = 1,line_length
                       Vn(iplan,alpha) =  
     &                    ((conjg(R(iplan,alpha))-T(alpha))
     &                    *Vn_1(iplan,alpha-1) 
     &                    +(S(iplan,alpha)-T(alpha))*
     &                     (Vn_1(iplan,alpha)+Vn_2(iplan,alpha-1))-
     &                     2.*Vn_2(iplan,alpha-1))/(R(iplan,alpha)+
     &                     T(alpha))
*
*         Transfert R -->  S for columns < n
*
                           S(iplan,alpha)=R(iplan,alpha)
*
*         Transfert Vn --> Vn_1 --> Vn_2 for columns < n-1
*
                           Vn_2(iplan,alpha-1) = Vn_1(iplan,alpha-1)
                           Vn_1(iplan,alpha-1) = Vn(iplan,alpha-1)
                  enddo
             enddo
*
*         Transfert Vn --> Vn_1 --> Vn_2  for column n-1
*
             do iplan = 1,line_length
                 Vn_2(iplan,n-1)=Vn_1(iplan,n-1)
                 Vn_1(iplan,n-1)=Vn(iplan,n-1)
             enddo
*
*         Transfert Vn --> Vn_1 for column n
*         Transfert R -->  S and initialisation of Vn  for  column n
*
            do iplan = 1,line_length
                Vn(iplan,n)=UN
                Vn_1(iplan,n)=Vn(iplan,n)
                S(iplan,n)=R(iplan,n)
             enddo
*
         enddo
*    
*     end main loop n: amplitude computed along exit surface in
*                      reciprocal geometry
*     Now compute the intensity for ONE point along the exit surface 
*     in the direct geometry
*
         do alpha=1,nm
             do iplan= 1,line_length
             intens(iplan,alpha) = ttrv(alpha)*
     &           (real(Vn(iplan,alpha))**2+aimag(Vn(iplan,alpha))**2)
*
*    The total intensity is computed using the rectangle method
*    A factor 0.5 is omitted
*
              stor(iplan) = stor(iplan) + (intens(iplan,alpha-1) +
     &                      intens(iplan,alpha))*transv(alpha) 
              enddo
         enddo
*
*    End loop on one plane of incidence
*    endloop iplan
*         
*    Now store image line in array out with offset
*
      do k = 1,image_size
          out(processor,k) = 0.
      enddo
      do k = 1,line_length
          out(processor,k+offset) = stor(k)
      enddo
      takagi =  0
      return
      end
